{% extends 'base.html' -%}
{% block body %}
<h2>404 - Page not found</h2>
<p>
    The page that you've requested doesn't exists or wasn't found.<br />
    Check the typed URL or go to our home page:
</p>
<p>
    <a href="{{ url_for('views.home', _external=True) }}">
        {{ url_for('views.home', _external=True) }}
    </a>
</p>
{% endblock %}
